@@ -3,19 +3,20 @@
 This module implements the Resource classes that translate JSON from Jira REST
 resources into usable objects.
 """
+
 from __future__ import annotations
 
 import json
 import logging
 import re
 import time
-from typing import TYPE_CHECKING, Any, Dict, List, Type, cast
+from typing import TYPE_CHECKING, Any, cast
 
 from requests import Response
 from requests.structures import CaseInsensitiveDict
 
 from jira.resilientsession import ResilientSession, parse_errors
-from jira.utils import json_loads, threaded_requests
+from jira.utils import json_loads, remove_empty_attributes, threaded_requests
 
 if TYPE_CHECKING:
     from jira.client import JIRA
@@ -37,7 +38,10 @@ __all__ = (
     "Attachment",
     "Component",
     "Dashboard",
+    "DashboardItemProperty",
+    "DashboardItemPropertyKey",
     "Filter",
+    "DashboardGadget",
     "Votes",
     "PermissionScheme",
     "Watchers",
@@ -65,6 +69,7 @@ __all__ = (
     "ServiceDesk",
     "RequestType",
     "resource_class_map",
+    "PinnedComment",
 )
 
 logging.getLogger("jira").addHandler(logging.NullHandler())
@@ -239,7 +244,7 @@ class Resource:
 
     def find(
         self,
-        id: tuple[str, str] | int | str,
+        id: tuple[str, ...] | int | str,
         params: dict[str, str] | None = None,
     ):
         """Finds a resource based on the input parameters.
@@ -291,7 +296,7 @@ class Resource:
         self,
         fields: dict[str, Any] | None = None,
         async_: bool | None = None,
-        jira: JIRA = None,
+        jira: JIRA | None = None,
         notify: bool = True,
         **kwargs: Any,
     ):
@@ -331,8 +336,9 @@ class Resource:
                 and "reporter" not in data["fields"]
             ):
                 logging.warning(
-                    "autofix: setting reporter to '%s' and retrying the update."
-                    % self._options["autofix"]
+                    "autofix: setting reporter to '{}' and retrying the update.".format(
+                        self._options["autofix"]
+                    )
                 )
                 data["fields"]["reporter"] = {"name": self._options["autofix"]}
 
@@ -381,8 +387,7 @@ class Resource:
 
             if user and jira:
                 logging.warning(
-                    "Trying to add missing orphan user '%s' in order to complete the previous failed operation."
-                    % user
+                    f"Trying to add missing orphan user '{user}' in order to complete the previous failed operation."
                 )
                 jira.add_user(user, "noreply@example.com", 10100, active=False)
                 # if 'assignee' not in data['fields']:
@@ -480,12 +485,12 @@ class Attachment(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "attachment/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
     def get(self):
         """Return the file content as a string."""
@@ -505,12 +510,12 @@ class Component(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "component/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
     def delete(self, moveIssuesTo: str | None = None):  # type: ignore[override]
         """Delete this component from the server.
@@ -532,12 +537,12 @@ class CustomFieldOption(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "customFieldOption/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Dashboard(Resource):
@@ -547,12 +552,179 @@ class Dashboard(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "dashboard/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.gadgets: list[DashboardGadget] = []
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
+
+
+class DashboardItemPropertyKey(Resource):
+    """A jira dashboard item property key."""
+
+    def __init__(
+        self,
+        options: dict[str, str],
+        session: ResilientSession,
+        raw: dict[str, Any] | None = None,
+    ):
+        Resource.__init__(self, "dashboard/{0}/items/{1}/properties", options, session)
+        if raw:
+            self._parse_raw(raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
+
+
+class DashboardItemProperty(Resource):
+    """A jira dashboard item."""
+
+    def __init__(
+        self,
+        options: dict[str, str],
+        session: ResilientSession,
+        raw: dict[str, Any] | None = None,
+    ):
+        Resource.__init__(
+            self, "dashboard/{0}/items/{1}/properties/{2}", options, session
+        )
+        if raw:
+            self._parse_raw(raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
+
+    def update(  # type: ignore[override] # incompatible supertype ignored
+        self, dashboard_id: str, item_id: str, value: dict[str, Any]
+    ) -> DashboardItemProperty:
+        """Update this resource on the server.
+
+        Keyword arguments are marshalled into a dict before being sent. If this resource doesn't support ``PUT``, a :py:exc:`.JIRAError`
+        will be raised; subclasses that specialize this method will only raise errors in case of user error.
+
+        Args:
+          dashboard_id (str): The ``id`` if the dashboard.
+          item_id (str): The id of the dashboard item (``DashboardGadget``) to target.
+          value (dict[str, Any]): The value of the targeted property key.
+
+        Returns:
+          DashboardItemProperty
+        """
+        options = self._options.copy()
+        options["path"] = (
+            f"dashboard/{dashboard_id}/items/{item_id}/properties/{self.key}"
+        )
+        self.raw["value"].update(value)
+        self._session.put(self.JIRA_BASE_URL.format(**options), self.raw["value"])
+
+        return DashboardItemProperty(self._options, self._session, raw=self.raw)
+
+    def delete(self, dashboard_id: str, item_id: str) -> Response:  # type: ignore[override] # incompatible supertype ignored
+        """Delete dashboard item property.
+
+        Args:
+          dashboard_id (str): The ``id`` of the dashboard.
+          item_id (str): The ``id`` of the dashboard item (``DashboardGadget``).
+
+
+        Returns:
+          Response
+        """
+        options = self._options.copy()
+        options["path"] = (
+            f"dashboard/{dashboard_id}/items/{item_id}/properties/{self.key}"
+        )
+
+        return self._session.delete(self.JIRA_BASE_URL.format(**options))
+
+
+class DashboardGadget(Resource):
+    """A jira dashboard gadget."""
+
+    def __init__(
+        self,
+        options: dict[str, str],
+        session: ResilientSession,
+        raw: dict[str, Any] | None = None,
+    ):
+        Resource.__init__(self, "dashboard/{0}/gadget/{1}", options, session)
+        if raw:
+            self._parse_raw(raw)
+        self.item_properties: list[DashboardItemProperty] = []
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
+
+    def update(  # type: ignore[override] # incompatible supertype ignored
+        self,
+        dashboard_id: str,
+        color: str | None = None,
+        position: dict[str, Any] | None = None,
+        title: str | None = None,
+    ) -> DashboardGadget:
+        """Update this resource on the server.
+
+        Keyword arguments are marshalled into a dict before being sent. If this resource doesn't support ``PUT``, a :py:exc:`.JIRAError`
+        will be raised; subclasses that specialize this method will only raise errors in case of user error.
+
+        Args:
+          dashboard_id (str): The ``id`` of the dashboard to add the gadget to `required`.
+          color (str): The color of the gadget, should be one of: blue, red, yellow,
+              green, cyan, purple, gray, or white.
+          ignore_uri_and_module_key_validation (bool): Whether to ignore the
+              validation of the module key and URI. For example, when a gadget is created
+              that is part of an application that is not installed.
+          position (dict[str, int]): A dictionary containing position information like -
+              `{"column": 0, "row", 1}`.
+          title (str): The title of the gadget.
+
+        Returns:
+          ``DashboardGadget``
+        """
+        data = remove_empty_attributes(
+            {"color": color, "position": position, "title": title}
+        )
+        options = self._options.copy()
+        options["path"] = f"dashboard/{dashboard_id}/gadget/{self.id}"
+
+        self._session.put(self.JIRA_BASE_URL.format(**options), json=data)
+        options["path"] = f"dashboard/{dashboard_id}/gadget"
+
+        return next(
+            DashboardGadget(self._options, self._session, raw=gadget)
+            for gadget in self._session.get(
+                self.JIRA_BASE_URL.format(**options)
+            ).json()["gadgets"]
+            if gadget["id"] == self.id
+        )
+
+    def delete(self, dashboard_id: str) -> Response:  # type: ignore[override] # incompatible supertype ignored
+        """Delete gadget from dashboard.
+
+        Args:
+          dashboard_id (str): The ``id`` of the dashboard.
+
+        Returns:
+          Response
+        """
+        options = self._options.copy()
+        options["path"] = f"dashboard/{dashboard_id}/gadget/{self.id}"
+
+        return self._session.delete(self.JIRA_BASE_URL.format(**options))
+
+
+class Field(Resource):
+    """An issue field.
+
+    A field cannot be fetched from the Jira API individually, but paginated lists of fields are returned by some endpoints.
+    """
+
+    def __init__(
+        self,
+        options: dict[str, str],
+        session: ResilientSession,
+        raw: dict[str, Any] | None = None,
+    ):
+        Resource.__init__(self, "field/{0}", options, session)
+        if raw:
+            self._parse_raw(raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Filter(Resource):
@@ -562,12 +734,12 @@ class Filter(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "filter/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Issue(Resource):
@@ -610,7 +782,7 @@ class Issue(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "issue/{0}", options, session)
 
@@ -619,14 +791,14 @@ class Issue(Resource):
         self.key: str
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
     def update(  # type: ignore[override] # incompatible supertype ignored
         self,
-        fields: dict[str, Any] = None,
-        update: dict[str, Any] = None,
-        async_: bool = None,
-        jira: JIRA = None,
+        fields: dict[str, Any] | None = None,
+        update: dict[str, Any] | None = None,
+        async_: bool | None = None,
+        jira: JIRA | None = None,
         notify: bool = True,
         **fieldargs,
     ):
@@ -735,12 +907,12 @@ class Comment(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "issue/{0}/comment/{1}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
     def update(  # type: ignore[override]
         # The above ignore is added because we've added new parameters and order of
@@ -749,7 +921,7 @@ class Comment(Resource):
         self,
         fields: dict[str, Any] | None = None,
         async_: bool | None = None,
-        jira: JIRA = None,
+        jira: JIRA | None = None,
         body: str = "",
         visibility: dict[str, str] | None = None,
         is_internal: bool = False,
@@ -784,6 +956,21 @@ class Comment(Resource):
         super().update(async_=async_, jira=jira, notify=notify, fields=data)
 
 
+class PinnedComment(Resource):
+    """Pinned comment on an issue."""
+
+    def __init__(
+        self,
+        options: dict[str, str],
+        session: ResilientSession,
+        raw: dict[str, Any] | None = None,
+    ):
+        Resource.__init__(self, "issue/{0}/pinned-comments", options, session)
+        if raw:
+            self._parse_raw(raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
+
+
 class RemoteLink(Resource):
     """A link to a remote application from an issue."""
 
@@ -791,14 +978,20 @@ class RemoteLink(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "issue/{0}/remotelink/{1}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
-    def update(self, object, globalId=None, application=None, relationship=None):
+    def update(  # type: ignore[override]
+        self,
+        object: dict[str, Any] | None,
+        globalId=None,
+        application=None,
+        relationship=None,
+    ):
         """Update a RemoteLink. 'object' is required.
 
         For definitions of the allowable fields for 'object' and the keyword arguments 'globalId', 'application' and 'relationship',
@@ -818,7 +1011,8 @@ class RemoteLink(Resource):
         if relationship is not None:
             data["relationship"] = relationship
 
-        super().update(**data)
+        # https://github.com/pycontribs/jira/issues/1881
+        super().update(**data)  # type: ignore[arg-type]
 
 
 class Votes(Resource):
@@ -828,12 +1022,12 @@ class Votes(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "issue/{0}/votes", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class IssueTypeScheme(Resource):
@@ -843,7 +1037,7 @@ class IssueTypeScheme(Resource):
         Resource.__init__(self, "issuetypescheme", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class IssueSecurityLevelScheme(Resource):
@@ -855,7 +1049,7 @@ class IssueSecurityLevelScheme(Resource):
         )
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class NotificationScheme(Resource):
@@ -867,7 +1061,7 @@ class NotificationScheme(Resource):
         )
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class PermissionScheme(Resource):
@@ -879,7 +1073,7 @@ class PermissionScheme(Resource):
         )
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class PriorityScheme(Resource):
@@ -891,7 +1085,7 @@ class PriorityScheme(Resource):
         )
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class WorkflowScheme(Resource):
@@ -903,7 +1097,7 @@ class WorkflowScheme(Resource):
         )
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Watchers(Resource):
@@ -913,14 +1107,14 @@ class Watchers(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "issue/{0}/watchers", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
-    def delete(self, username):
+    def delete(self, username):  # type: ignore[override]
         """Remove the specified user from the watchers list."""
         super().delete(params={"username": username})
 
@@ -930,13 +1124,13 @@ class TimeTracking(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "issue/{0}/worklog/{1}", options, session)
         self.remainingEstimate = None
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Worklog(Resource):
@@ -946,12 +1140,12 @@ class Worklog(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "issue/{0}/worklog/{1}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
     def delete(  # type: ignore[override]
         self, adjustEstimate: str | None = None, newEstimate=None, increaseBy=None
@@ -983,12 +1177,12 @@ class IssueProperty(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "issue/{0}/properties/{1}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
     def _find_by_url(
         self,
@@ -1007,12 +1201,12 @@ class IssueLink(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "issueLink/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class IssueLinkType(Resource):
@@ -1022,12 +1216,12 @@ class IssueLinkType(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "issueLinkType/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class IssueType(Resource):
@@ -1037,12 +1231,12 @@ class IssueType(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "issuetype/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Priority(Resource):
@@ -1052,12 +1246,12 @@ class Priority(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "priority/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Project(Resource):
@@ -1067,12 +1261,12 @@ class Project(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "project/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Role(Resource):
@@ -1082,17 +1276,17 @@ class Role(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "project/{0}/role/{1}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
     def update(  # type: ignore[override]
         self,
-        users: str | list | tuple = None,
-        groups: str | list | tuple = None,
+        users: str | list | tuple | None = None,
+        groups: str | list | tuple | None = None,
     ):
         """Add the specified users or groups to this project role. One of ``users`` or ``groups`` must be specified.
 
@@ -1117,8 +1311,8 @@ class Role(Resource):
 
     def add_user(
         self,
-        users: str | list | tuple = None,
-        groups: str | list | tuple = None,
+        users: str | list | tuple | None = None,
+        groups: str | list | tuple | None = None,
     ):
         """Add the specified users or groups to this project role. One of ``users`` or ``groups`` must be specified.
 
@@ -1142,12 +1336,12 @@ class Resolution(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "resolution/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class SecurityLevel(Resource):
@@ -1157,12 +1351,12 @@ class SecurityLevel(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "securitylevel/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Status(Resource):
@@ -1172,12 +1366,12 @@ class Status(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "status/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class StatusCategory(Resource):
@@ -1187,12 +1381,12 @@ class StatusCategory(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "statuscategory/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class User(Resource):
@@ -1202,7 +1396,7 @@ class User(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
         *,
         _query_param: str = "username",
     ):
@@ -1213,7 +1407,7 @@ class User(Resource):
         Resource.__init__(self, f"user?{_query_param}" + "={0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Group(Resource):
@@ -1223,12 +1417,12 @@ class Group(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "group?groupname={0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Version(Resource):
@@ -1238,12 +1432,12 @@ class Version(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "version/{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
     def delete(self, moveFixIssuesTo=None, moveAffectedIssuesTo=None):
         """Delete this project version from the server.
@@ -1262,7 +1456,8 @@ class Version(Resource):
 
         return super().delete(params)
 
-    def update(self, **kwargs):
+    # TODO: https://github.com/pycontribs/jira/issues/1881
+    def update(self, **kwargs):  # type: ignore[override]
         """Update this project version from the server. It is prior used to archive versions.
 
         Refer to Atlassian REST API `documentation`_.
@@ -1307,14 +1502,14 @@ class AgileResource(Resource):
         path: str,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         self.self = None
 
         Resource.__init__(self, path, options, session, self.AGILE_BASE_URL)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class Sprint(AgileResource):
@@ -1324,7 +1519,7 @@ class Sprint(AgileResource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         AgileResource.__init__(self, "sprint/{0}", options, session, raw)
 
@@ -1336,7 +1531,7 @@ class Board(AgileResource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         AgileResource.__init__(self, "board/{id}", options, session, raw)
 
@@ -1351,14 +1546,14 @@ class Customer(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(
             self, "customer", options, session, "{server}/rest/servicedeskapi/{path}"
         )
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class ServiceDesk(Resource):
@@ -1368,7 +1563,7 @@ class ServiceDesk(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(
             self,
@@ -1379,7 +1574,7 @@ class ServiceDesk(Resource):
         )
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 class RequestType(Resource):
@@ -1389,7 +1584,7 @@ class RequestType(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(
             self,
@@ -1401,7 +1596,7 @@ class RequestType(Resource):
 
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 # Utilities
@@ -1423,11 +1618,13 @@ def dict2resource(
         if isinstance(j, dict):
             if "self" in j:
                 # to try and help mypy know that cls_for_resource can never be 'Resource'
-                resource_class = cast(Type[Resource], cls_for_resource(j["self"]))
+                resource_class = cast(type[Resource], cls_for_resource(j["self"]))
                 resource = cast(
-                    Type[Resource],
+                    type[Resource],
                     resource_class(  # type: ignore
-                        options=options, session=session, raw=j  # type: ignore
+                        options=options,
+                        session=session,
+                        raw=j,  # type: ignore
                     ),
                 )
                 setattr(top, i, resource)
@@ -1436,17 +1633,17 @@ def dict2resource(
             else:
                 setattr(top, i, dict2resource(j, options=options, session=session))
         elif isinstance(j, seqs):
-            j = cast(List[Dict[str, Any]], j)  # help mypy
+            j = cast(list[dict[str, Any]], j)  # help mypy
             seq_list: list[Any] = []
             for seq_elem in j:
                 if isinstance(seq_elem, dict):
                     if "self" in seq_elem:
                         # to try and help mypy know that cls_for_resource can never be 'Resource'
                         resource_class = cast(
-                            Type[Resource], cls_for_resource(seq_elem["self"])
+                            type[Resource], cls_for_resource(seq_elem["self"])
                         )
                         resource = cast(
-                            Type[Resource],
+                            type[Resource],
                             resource_class(  # type: ignore
                                 options=options,
                                 session=session,
@@ -1472,9 +1669,13 @@ resource_class_map: dict[str, type[Resource]] = {
     r"component/[^/]+$": Component,
     r"customFieldOption/[^/]+$": CustomFieldOption,
     r"dashboard/[^/]+$": Dashboard,
+    r"dashboard/[^/]+/items/[^/]+/properties+$": DashboardItemPropertyKey,
+    r"dashboard/[^/]+/items/[^/]+/properties/[^/]+$": DashboardItemProperty,
+    r"dashboard/[^/]+/gadget/[^/]+$": DashboardGadget,
     r"filter/[^/]$": Filter,
     r"issue/[^/]+$": Issue,
     r"issue/[^/]+/comment/[^/]+$": Comment,
+    r"issue/[^/]+/pinned-comments$": PinnedComment,
     r"issue/[^/]+/votes$": Votes,
     r"issue/[^/]+/watchers$": Watchers,
     r"issue/[^/]+/worklog/[^/]+$": Worklog,
@@ -1511,12 +1712,12 @@ class UnknownResource(Resource):
         self,
         options: dict[str, str],
         session: ResilientSession,
-        raw: dict[str, Any] = None,
+        raw: dict[str, Any] | None = None,
     ):
         Resource.__init__(self, "unknown{0}", options, session)
         if raw:
             self._parse_raw(raw)
-        self.raw: dict[str, Any] = cast(Dict[str, Any], self.raw)
+        self.raw: dict[str, Any] = cast(dict[str, Any], self.raw)
 
 
 def cls_for_resource(resource_literal: str) -> type[Resource]:
