@@ -8,6 +8,7 @@ Refer to conftest.py for shared helper methods.
 resources/test_* : For tests related to resources
 test_* : For other tests of the non-resource elements of the jira package.
 """
+
 from __future__ import annotations
 
 import logging
@@ -22,7 +23,6 @@ import requests
 from parameterized import parameterized
 
 from jira import JIRA, Issue, JIRAError
-from jira.client import ResultList
 from jira.resources import Dashboard, Resource, cls_for_resource
 from tests.conftest import JiraTestCase, allow_on_cloud, rndpassword
 
@@ -230,11 +230,17 @@ class SearchTests(JiraTestCase):
 
     def test_search_issues(self):
         issues = self.jira.search_issues(f"project={self.project_b}")
-        issues = cast(ResultList[Issue], issues)
         self.assertLessEqual(len(issues), 50)  # default maxResults
         for issue in issues:
             self.assertTrue(issue.key.startswith(self.project_b))
 
+    def test_search_issues_json(self):
+        result = self.jira.search_issues(f"project={self.project_b}", json_result=True)
+        issues = result["issues"]
+        self.assertLessEqual(len(issues), 50)  # default maxResults
+        for issue in issues:
+            self.assertTrue(issue["key"].startswith(self.project_b))
+
     def test_search_issues_async(self):
         original_val = self.jira._options["async"]
         try:
@@ -242,7 +248,6 @@ class SearchTests(JiraTestCase):
             issues = self.jira.search_issues(
                 f"project={self.project_b}", maxResults=False
             )
-            issues = cast(ResultList[Issue], issues)
             self.assertEqual(len(issues), issues.total)
             for issue in issues:
                 self.assertTrue(issue.key.startswith(self.project_b))
@@ -262,7 +267,6 @@ class SearchTests(JiraTestCase):
 
     def test_search_issues_field_limiting(self):
         issues = self.jira.search_issues(f"key={self.issue}", fields="summary,comment")
-        issues = cast(ResultList[Issue], issues)
         self.assertTrue(hasattr(issues[0].fields, "summary"))
         self.assertTrue(hasattr(issues[0].fields, "comment"))
         self.assertFalse(hasattr(issues[0].fields, "reporter"))
@@ -270,13 +274,20 @@ class SearchTests(JiraTestCase):
 
     def test_search_issues_expand(self):
         issues = self.jira.search_issues(f"key={self.issue}", expand="changelog")
-        issues = cast(ResultList[Issue], issues)
         # self.assertTrue(hasattr(issues[0], 'names'))
         self.assertEqual(len(issues), 1)
         self.assertFalse(hasattr(issues[0], "editmeta"))
         self.assertTrue(hasattr(issues[0], "changelog"))
         self.assertEqual(issues[0].key, self.issue)
 
+    def test_search_issues_use_post(self):
+        long_jql = " or ".join(f"key={self.issue}" for _ in range(2000))
+        with pytest.raises(JIRAError):
+            self.jira.search_issues(long_jql)
+        issues = self.jira.search_issues(long_jql, use_post=True)
+        self.assertEqual(len(issues), 1)
+        self.assertEqual(issues[0].key, self.issue)
+
 
 class ServerInfoTests(JiraTestCase):
     def test_server_info(self):
@@ -472,7 +483,7 @@ def _calculate_calls_for_fetch_pages(
     total: int,
     max_results: int,
     batch_size: int | None,
-    default: int | None = 10,
+    default: int = 10,
 ):
     """Returns expected query parameters for specified search-issues arguments."""
     if not max_results:
